import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

/// Clasificamos de manera personalizada, para utilizarlo despues
/// Tipos de animaciones
enum AnimationType { normal, fadeIn }

/// Main class, [context] es el BuildContext de la aplicacion en ese momento
/// [child] es el widget a navegar,
/// [animation] es el tipo de animación,
/// [duration] es el tiempo quedemorara en realizar la animacion
/// [replacement] es un indicador, si queremos retornar a la pagina anterior o no
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  /// Duration duration
  /// }):this.duration = duration ?? Duration(milliseconds: 300){
  /// Si no mencionamos quees una constante no nos votara error,  sera constante siempre y cuando no enviemos dicho campo
  RouteTransitions(
      {@required this.context,
      @required this.child,
      this.animation = AnimationType.normal,
      this.duration = const Duration(milliseconds: 300),
      this.replacement = false}) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        this._fadeInTransition();
        break;
    }
  }

  /// Push normal de la página
  /// Push normal of the page
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Push replacement de la página
  /// Push replacement of the page
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  /// Código de una transición normal
  /// Code of a normal transition
  /// Widget de ruta simple, indicando el el page widget
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Controlador de la transición con fadeIn
  /// Controller of the trnasition with fadeIn
  /// Toma el child de aqui //Widget de ruta compuesta, aparte de indicar el page widget incluye las transicion que queremos añadir
  void _fadeInTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (BuildContext context, _, __) => this.child,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
              child: child,
              opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
                  CurvedAnimation(curve: Curves.easeInOut, parent: animation)));
        });
    (this.replacement) ? _pushPage(route) : _pushReplacementPage(route);
  }
}
